# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_resource_claim import V1ResourceClaim

class V1ResourceRequirements(BaseModel):
    """
    ResourceRequirements describes the compute resource requirements.  # noqa: E501
    """
    claims: Optional[conlist(V1ResourceClaim)] = Field(None, description="Claims lists the names of resources, defined in spec.resourceClaims, that are used by this container.  This is an alpha field and requires enabling the DynamicResourceAllocation feature gate.  This field is immutable. It can only be set for containers.")
    limits: Optional[Dict[str, StrictStr]] = Field(None, description="Limits describes the maximum amount of compute resources allowed. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/")
    requests: Optional[Dict[str, StrictStr]] = Field(None, description="Requests describes the minimum amount of compute resources required. If Requests is omitted for a container, it defaults to Limits if that is explicitly specified, otherwise to an implementation-defined value. Requests cannot exceed Limits. More info: https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/")
    __properties = ["claims", "limits", "requests"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ResourceRequirements:
        """Create an instance of V1ResourceRequirements from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in claims (list)
        _items = []
        if self.claims:
            for _item in self.claims:
                if _item:
                    _items.append(_item.to_dict())
            _dict['claims'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ResourceRequirements:
        """Create an instance of V1ResourceRequirements from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ResourceRequirements.parse_obj(obj)

        _obj = V1ResourceRequirements.parse_obj({
            "claims": [V1ResourceClaim.from_dict(_item) for _item in obj.get("claims")] if obj.get("claims") is not None else None,
            "limits": obj.get("limits"),
            "requests": obj.get("requests")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field
from .v1_custom_resource_subresource_scale import V1CustomResourceSubresourceScale

class V1CustomResourceSubresources(BaseModel):
    """
    CustomResourceSubresources defines the status and scale subresources for CustomResources.  # noqa: E501
    """
    scale: Optional[V1CustomResourceSubresourceScale] = None
    status: Optional[Dict[str, Any]] = Field(None, description="status indicates the custom resource should serve a `/status` subresource. When enabled: 1. requests to the custom resource primary endpoint ignore changes to the `status` stanza of the object. 2. requests to the custom resource `/status` subresource ignore changes to anything other than the `status` stanza of the object.")
    __properties = ["scale", "status"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1CustomResourceSubresources:
        """Create an instance of V1CustomResourceSubresources from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of scale
        if self.scale:
            _dict['scale'] = self.scale.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1CustomResourceSubresources:
        """Create an instance of V1CustomResourceSubresources from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1CustomResourceSubresources.parse_obj(obj)

        _obj = V1CustomResourceSubresources.parse_obj({
            "scale": V1CustomResourceSubresourceScale.from_dict(obj.get("scale")) if obj.get("scale") is not None else None,
            "status": obj.get("status")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from .v1_key_to_path import V1KeyToPath

class V1SecretVolumeSource(BaseModel):
    """
    Adapts a Secret into a volume.  The contents of the target Secret's Data field will be presented in a volume as files using the keys in the Data field as the file names. Secret volumes support ownership management and SELinux relabeling.  # noqa: E501
    """
    default_mode: Optional[StrictInt] = Field(None, alias="defaultMode", description="defaultMode is Optional: mode bits used to set permissions on created files by default. Must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. Defaults to 0644. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.")
    items: Optional[conlist(V1KeyToPath)] = Field(None, description="items If unspecified, each key-value pair in the Data field of the referenced Secret will be projected into the volume as a file whose name is the key and content is the value. If specified, the listed keys will be projected into the specified paths, and unlisted keys will not be present. If a key is specified which is not present in the Secret, the volume setup will error unless it is marked optional. Paths must be relative and may not contain the '..' path or start with '..'.")
    optional: Optional[StrictBool] = Field(None, description="optional field specify whether the Secret or its keys must be defined")
    secret_name: Optional[StrictStr] = Field(None, alias="secretName", description="secretName is the name of the secret in the pod's namespace to use. More info: https://kubernetes.io/docs/concepts/storage/volumes#secret")
    __properties = ["defaultMode", "items", "optional", "secretName"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1SecretVolumeSource:
        """Create an instance of V1SecretVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in items (list)
        _items = []
        if self.items:
            for _item in self.items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['items'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1SecretVolumeSource:
        """Create an instance of V1SecretVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1SecretVolumeSource.parse_obj(obj)

        _obj = V1SecretVolumeSource.parse_obj({
            "default_mode": obj.get("defaultMode"),
            "items": [V1KeyToPath.from_dict(_item) for _item in obj.get("items")] if obj.get("items") is not None else None,
            "optional": obj.get("optional"),
            "secret_name": obj.get("secretName")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist

class V1beta3ResourcePolicyRule(BaseModel):
    """
    ResourcePolicyRule is a predicate that matches some resource requests, testing the request's verb and the target resource. A ResourcePolicyRule matches a resource request if and only if: (a) at least one member of verbs matches the request, (b) at least one member of apiGroups matches the request, (c) at least one member of resources matches the request, and (d) either (d1) the request does not specify a namespace (i.e., `Namespace==\"\"`) and clusterScope is true or (d2) the request specifies a namespace and least one member of namespaces matches the request's namespace.  # noqa: E501
    """
    api_groups: conlist(StrictStr) = Field(..., alias="apiGroups", description="`apiGroups` is a list of matching API groups and may not be empty. \"*\" matches all API groups and, if present, must be the only entry. Required.")
    cluster_scope: Optional[StrictBool] = Field(None, alias="clusterScope", description="`clusterScope` indicates whether to match requests that do not specify a namespace (which happens either because the resource is not namespaced or the request targets all namespaces). If this field is omitted or false then the `namespaces` field must contain a non-empty list.")
    namespaces: Optional[conlist(StrictStr)] = Field(None, description="`namespaces` is a list of target namespaces that restricts matches.  A request that specifies a target namespace matches only if either (a) this list contains that target namespace or (b) this list contains \"*\".  Note that \"*\" matches any specified namespace but does not match a request that _does not specify_ a namespace (see the `clusterScope` field for that). This list may be empty, but only if `clusterScope` is true.")
    resources: conlist(StrictStr) = Field(..., description="`resources` is a list of matching resources (i.e., lowercase and plural) with, if desired, subresource.  For example, [ \"services\", \"nodes/status\" ].  This list may not be empty. \"*\" matches all resources and, if present, must be the only entry. Required.")
    verbs: conlist(StrictStr) = Field(..., description="`verbs` is a list of matching verbs and may not be empty. \"*\" matches all verbs and, if present, must be the only entry. Required.")
    __properties = ["apiGroups", "clusterScope", "namespaces", "resources", "verbs"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1beta3ResourcePolicyRule:
        """Create an instance of V1beta3ResourcePolicyRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1beta3ResourcePolicyRule:
        """Create an instance of V1beta3ResourcePolicyRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1beta3ResourcePolicyRule.parse_obj(obj)

        _obj = V1beta3ResourcePolicyRule.parse_obj({
            "api_groups": obj.get("apiGroups"),
            "cluster_scope": obj.get("clusterScope"),
            "namespaces": obj.get("namespaces"),
            "resources": obj.get("resources"),
            "verbs": obj.get("verbs")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from .v1_se_linux_options import V1SELinuxOptions
from .v1_seccomp_profile import V1SeccompProfile
from .v1_sysctl import V1Sysctl
from .v1_windows_security_context_options import V1WindowsSecurityContextOptions

class V1PodSecurityContext(BaseModel):
    """
    PodSecurityContext holds pod-level security attributes and common container settings. Some fields are also present in container.securityContext.  Field values of container.securityContext take precedence over field values of PodSecurityContext.  # noqa: E501
    """
    fs_group: Optional[StrictInt] = Field(None, alias="fsGroup", description="A special supplemental group that applies to all containers in a pod. Some volume types allow the Kubelet to change the ownership of that volume to be owned by the pod:  1. The owning GID will be the FSGroup 2. The setgid bit is set (new files created in the volume will be owned by FSGroup) 3. The permission bits are OR'd with rw-rw----  If unset, the Kubelet will not modify the ownership and permissions of any volume. Note that this field cannot be set when spec.os.name is windows.")
    fs_group_change_policy: Optional[StrictStr] = Field(None, alias="fsGroupChangePolicy", description="fsGroupChangePolicy defines behavior of changing ownership and permission of the volume before being exposed inside Pod. This field will only apply to volume types which support fsGroup based ownership(and permissions). It will have no effect on ephemeral volume types such as: secret, configmaps and emptydir. Valid values are \"OnRootMismatch\" and \"Always\". If not specified, \"Always\" is used. Note that this field cannot be set when spec.os.name is windows.")
    run_as_group: Optional[StrictInt] = Field(None, alias="runAsGroup", description="The GID to run the entrypoint of the container process. Uses runtime default if unset. May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence for that container. Note that this field cannot be set when spec.os.name is windows.")
    run_as_non_root: Optional[StrictBool] = Field(None, alias="runAsNonRoot", description="Indicates that the container must run as a non-root user. If true, the Kubelet will validate the image at runtime to ensure that it does not run as UID 0 (root) and fail to start the container if it does. If unset or false, no such validation will be performed. May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.")
    run_as_user: Optional[StrictInt] = Field(None, alias="runAsUser", description="The UID to run the entrypoint of the container process. Defaults to user specified in image metadata if unspecified. May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence for that container. Note that this field cannot be set when spec.os.name is windows.")
    se_linux_options: Optional[V1SELinuxOptions] = Field(None, alias="seLinuxOptions")
    seccomp_profile: Optional[V1SeccompProfile] = Field(None, alias="seccompProfile")
    supplemental_groups: Optional[conlist(StrictInt)] = Field(None, alias="supplementalGroups", description="A list of groups applied to the first process run in each container, in addition to the container's primary GID, the fsGroup (if specified), and group memberships defined in the container image for the uid of the container process. If unspecified, no additional groups are added to any container. Note that group memberships defined in the container image for the uid of the container process are still effective, even if they are not included in this list. Note that this field cannot be set when spec.os.name is windows.")
    sysctls: Optional[conlist(V1Sysctl)] = Field(None, description="Sysctls hold a list of namespaced sysctls used for the pod. Pods with unsupported sysctls (by the container runtime) might fail to launch. Note that this field cannot be set when spec.os.name is windows.")
    windows_options: Optional[V1WindowsSecurityContextOptions] = Field(None, alias="windowsOptions")
    __properties = ["fsGroup", "fsGroupChangePolicy", "runAsGroup", "runAsNonRoot", "runAsUser", "seLinuxOptions", "seccompProfile", "supplementalGroups", "sysctls", "windowsOptions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodSecurityContext:
        """Create an instance of V1PodSecurityContext from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of se_linux_options
        if self.se_linux_options:
            _dict['seLinuxOptions'] = self.se_linux_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of seccomp_profile
        if self.seccomp_profile:
            _dict['seccompProfile'] = self.seccomp_profile.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in sysctls (list)
        _items = []
        if self.sysctls:
            for _item in self.sysctls:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sysctls'] = _items
        # override the default output from pydantic by calling `to_dict()` of windows_options
        if self.windows_options:
            _dict['windowsOptions'] = self.windows_options.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodSecurityContext:
        """Create an instance of V1PodSecurityContext from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodSecurityContext.parse_obj(obj)

        _obj = V1PodSecurityContext.parse_obj({
            "fs_group": obj.get("fsGroup"),
            "fs_group_change_policy": obj.get("fsGroupChangePolicy"),
            "run_as_group": obj.get("runAsGroup"),
            "run_as_non_root": obj.get("runAsNonRoot"),
            "run_as_user": obj.get("runAsUser"),
            "se_linux_options": V1SELinuxOptions.from_dict(obj.get("seLinuxOptions")) if obj.get("seLinuxOptions") is not None else None,
            "seccomp_profile": V1SeccompProfile.from_dict(obj.get("seccompProfile")) if obj.get("seccompProfile") is not None else None,
            "supplemental_groups": obj.get("supplementalGroups"),
            "sysctls": [V1Sysctl.from_dict(_item) for _item in obj.get("sysctls")] if obj.get("sysctls") is not None else None,
            "windows_options": V1WindowsSecurityContextOptions.from_dict(obj.get("windowsOptions")) if obj.get("windowsOptions") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v1_config_map_key_selector import V1ConfigMapKeySelector
from .v1_object_field_selector import V1ObjectFieldSelector
from .v1_resource_field_selector import V1ResourceFieldSelector
from .v1_secret_key_selector import V1SecretKeySelector

class V1EnvVarSource(BaseModel):
    """
    EnvVarSource represents a source for the value of an EnvVar.  # noqa: E501
    """
    config_map_key_ref: Optional[V1ConfigMapKeySelector] = Field(None, alias="configMapKeyRef")
    field_ref: Optional[V1ObjectFieldSelector] = Field(None, alias="fieldRef")
    resource_field_ref: Optional[V1ResourceFieldSelector] = Field(None, alias="resourceFieldRef")
    secret_key_ref: Optional[V1SecretKeySelector] = Field(None, alias="secretKeyRef")
    __properties = ["configMapKeyRef", "fieldRef", "resourceFieldRef", "secretKeyRef"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1EnvVarSource:
        """Create an instance of V1EnvVarSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of config_map_key_ref
        if self.config_map_key_ref:
            _dict['configMapKeyRef'] = self.config_map_key_ref.to_dict()
        # override the default output from pydantic by calling `to_dict()` of field_ref
        if self.field_ref:
            _dict['fieldRef'] = self.field_ref.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource_field_ref
        if self.resource_field_ref:
            _dict['resourceFieldRef'] = self.resource_field_ref.to_dict()
        # override the default output from pydantic by calling `to_dict()` of secret_key_ref
        if self.secret_key_ref:
            _dict['secretKeyRef'] = self.secret_key_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1EnvVarSource:
        """Create an instance of V1EnvVarSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1EnvVarSource.parse_obj(obj)

        _obj = V1EnvVarSource.parse_obj({
            "config_map_key_ref": V1ConfigMapKeySelector.from_dict(obj.get("configMapKeyRef")) if obj.get("configMapKeyRef") is not None else None,
            "field_ref": V1ObjectFieldSelector.from_dict(obj.get("fieldRef")) if obj.get("fieldRef") is not None else None,
            "resource_field_ref": V1ResourceFieldSelector.from_dict(obj.get("resourceFieldRef")) if obj.get("resourceFieldRef") is not None else None,
            "secret_key_ref": V1SecretKeySelector.from_dict(obj.get("secretKeyRef")) if obj.get("secretKeyRef") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt
from .v1_label_selector import V1LabelSelector
from .v1_pod_template_spec import V1PodTemplateSpec

class V1ReplicaSetSpec(BaseModel):
    """
    ReplicaSetSpec is the specification of a ReplicaSet.  # noqa: E501
    """
    min_ready_seconds: Optional[StrictInt] = Field(None, alias="minReadySeconds", description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)")
    replicas: Optional[StrictInt] = Field(None, description="Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller")
    selector: V1LabelSelector = Field(...)
    template: Optional[V1PodTemplateSpec] = None
    __properties = ["minReadySeconds", "replicas", "selector", "template"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ReplicaSetSpec:
        """Create an instance of V1ReplicaSetSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of selector
        if self.selector:
            _dict['selector'] = self.selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of template
        if self.template:
            _dict['template'] = self.template.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ReplicaSetSpec:
        """Create an instance of V1ReplicaSetSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ReplicaSetSpec.parse_obj(obj)

        _obj = V1ReplicaSetSpec.parse_obj({
            "min_ready_seconds": obj.get("minReadySeconds"),
            "replicas": obj.get("replicas"),
            "selector": V1LabelSelector.from_dict(obj.get("selector")) if obj.get("selector") is not None else None,
            "template": V1PodTemplateSpec.from_dict(obj.get("template")) if obj.get("template") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, conlist
from .v1_node_selector import V1NodeSelector
from .v1alpha2_resource_handle import V1alpha2ResourceHandle

class V1alpha2AllocationResult(BaseModel):
    """
    AllocationResult contains attributes of an allocated resource.  # noqa: E501
    """
    available_on_nodes: Optional[V1NodeSelector] = Field(None, alias="availableOnNodes")
    resource_handles: Optional[conlist(V1alpha2ResourceHandle)] = Field(None, alias="resourceHandles", description="ResourceHandles contain the state associated with an allocation that should be maintained throughout the lifetime of a claim. Each ResourceHandle contains data that should be passed to a specific kubelet plugin once it lands on a node. This data is returned by the driver after a successful allocation and is opaque to Kubernetes. Driver documentation may explain to users how to interpret this data if needed.  Setting this field is optional. It has a maximum size of 32 entries. If null (or empty), it is assumed this allocation will be processed by a single kubelet plugin with no ResourceHandle data attached. The name of the kubelet plugin invoked will match the DriverName set in the ResourceClaimStatus this AllocationResult is embedded in.")
    shareable: Optional[StrictBool] = Field(None, description="Shareable determines whether the resource supports more than one consumer at a time.")
    __properties = ["availableOnNodes", "resourceHandles", "shareable"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha2AllocationResult:
        """Create an instance of V1alpha2AllocationResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of available_on_nodes
        if self.available_on_nodes:
            _dict['availableOnNodes'] = self.available_on_nodes.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in resource_handles (list)
        _items = []
        if self.resource_handles:
            for _item in self.resource_handles:
                if _item:
                    _items.append(_item.to_dict())
            _dict['resourceHandles'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha2AllocationResult:
        """Create an instance of V1alpha2AllocationResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha2AllocationResult.parse_obj(obj)

        _obj = V1alpha2AllocationResult.parse_obj({
            "available_on_nodes": V1NodeSelector.from_dict(obj.get("availableOnNodes")) if obj.get("availableOnNodes") is not None else None,
            "resource_handles": [V1alpha2ResourceHandle.from_dict(_item) for _item in obj.get("resourceHandles")] if obj.get("resourceHandles") is not None else None,
            "shareable": obj.get("shareable")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr

class V1VolumeMount(BaseModel):
    """
    VolumeMount describes a mounting of a Volume within a container.  # noqa: E501
    """
    mount_path: StrictStr = Field(..., alias="mountPath", description="Path within the container at which the volume should be mounted.  Must not contain ':'.")
    mount_propagation: Optional[StrictStr] = Field(None, alias="mountPropagation", description="mountPropagation determines how mounts are propagated from the host to container and the other way around. When not set, MountPropagationNone is used. This field is beta in 1.10.")
    name: StrictStr = Field(..., description="This must match the Name of a Volume.")
    read_only: Optional[StrictBool] = Field(None, alias="readOnly", description="Mounted read-only if true, read-write otherwise (false or unspecified). Defaults to false.")
    sub_path: Optional[StrictStr] = Field(None, alias="subPath", description="Path within the volume from which the container's volume should be mounted. Defaults to \"\" (volume's root).")
    sub_path_expr: Optional[StrictStr] = Field(None, alias="subPathExpr", description="Expanded path within the volume from which the container's volume should be mounted. Behaves similarly to SubPath but environment variable references $(VAR_NAME) are expanded using the container's environment. Defaults to \"\" (volume's root). SubPathExpr and SubPath are mutually exclusive.")
    __properties = ["mountPath", "mountPropagation", "name", "readOnly", "subPath", "subPathExpr"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1VolumeMount:
        """Create an instance of V1VolumeMount from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1VolumeMount:
        """Create an instance of V1VolumeMount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1VolumeMount.parse_obj(obj)

        _obj = V1VolumeMount.parse_obj({
            "mount_path": obj.get("mountPath"),
            "mount_propagation": obj.get("mountPropagation"),
            "name": obj.get("name"),
            "read_only": obj.get("readOnly"),
            "sub_path": obj.get("subPath"),
            "sub_path_expr": obj.get("subPathExpr")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v1_non_resource_attributes import V1NonResourceAttributes
from .v1_resource_attributes import V1ResourceAttributes

class V1SelfSubjectAccessReviewSpec(BaseModel):
    """
    SelfSubjectAccessReviewSpec is a description of the access request.  Exactly one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must be set  # noqa: E501
    """
    non_resource_attributes: Optional[V1NonResourceAttributes] = Field(None, alias="nonResourceAttributes")
    resource_attributes: Optional[V1ResourceAttributes] = Field(None, alias="resourceAttributes")
    __properties = ["nonResourceAttributes", "resourceAttributes"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1SelfSubjectAccessReviewSpec:
        """Create an instance of V1SelfSubjectAccessReviewSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of non_resource_attributes
        if self.non_resource_attributes:
            _dict['nonResourceAttributes'] = self.non_resource_attributes.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource_attributes
        if self.resource_attributes:
            _dict['resourceAttributes'] = self.resource_attributes.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1SelfSubjectAccessReviewSpec:
        """Create an instance of V1SelfSubjectAccessReviewSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1SelfSubjectAccessReviewSpec.parse_obj(obj)

        _obj = V1SelfSubjectAccessReviewSpec.parse_obj({
            "non_resource_attributes": V1NonResourceAttributes.from_dict(obj.get("nonResourceAttributes")) if obj.get("nonResourceAttributes") is not None else None,
            "resource_attributes": V1ResourceAttributes.from_dict(obj.get("resourceAttributes")) if obj.get("resourceAttributes") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_exempt_priority_level_configuration import V1ExemptPriorityLevelConfiguration
from .v1_limited_priority_level_configuration import V1LimitedPriorityLevelConfiguration

class V1PriorityLevelConfigurationSpec(BaseModel):
    """
    PriorityLevelConfigurationSpec specifies the configuration of a priority level.  # noqa: E501
    """
    exempt: Optional[V1ExemptPriorityLevelConfiguration] = None
    limited: Optional[V1LimitedPriorityLevelConfiguration] = None
    type: StrictStr = Field(..., description="`type` indicates whether this priority level is subject to limitation on request execution.  A value of `\"Exempt\"` means that requests of this priority level are not subject to a limit (and thus are never queued) and do not detract from the capacity made available to other priority levels.  A value of `\"Limited\"` means that (a) requests of this priority level _are_ subject to limits and (b) some of the server's limited capacity is made available exclusively to this priority level. Required.")
    __properties = ["exempt", "limited", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PriorityLevelConfigurationSpec:
        """Create an instance of V1PriorityLevelConfigurationSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of exempt
        if self.exempt:
            _dict['exempt'] = self.exempt.to_dict()
        # override the default output from pydantic by calling `to_dict()` of limited
        if self.limited:
            _dict['limited'] = self.limited.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PriorityLevelConfigurationSpec:
        """Create an instance of V1PriorityLevelConfigurationSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PriorityLevelConfigurationSpec.parse_obj(obj)

        _obj = V1PriorityLevelConfigurationSpec.parse_obj({
            "exempt": V1ExemptPriorityLevelConfiguration.from_dict(obj.get("exempt")) if obj.get("exempt") is not None else None,
            "limited": V1LimitedPriorityLevelConfiguration.from_dict(obj.get("limited")) if obj.get("limited") is not None else None,
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel
from .v1_container_state_running import V1ContainerStateRunning
from .v1_container_state_terminated import V1ContainerStateTerminated
from .v1_container_state_waiting import V1ContainerStateWaiting

class V1ContainerState(BaseModel):
    """
    ContainerState holds a possible state of container. Only one of its members may be specified. If none of them is specified, the default one is ContainerStateWaiting.  # noqa: E501
    """
    running: Optional[V1ContainerStateRunning] = None
    terminated: Optional[V1ContainerStateTerminated] = None
    waiting: Optional[V1ContainerStateWaiting] = None
    __properties = ["running", "terminated", "waiting"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ContainerState:
        """Create an instance of V1ContainerState from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of running
        if self.running:
            _dict['running'] = self.running.to_dict()
        # override the default output from pydantic by calling `to_dict()` of terminated
        if self.terminated:
            _dict['terminated'] = self.terminated.to_dict()
        # override the default output from pydantic by calling `to_dict()` of waiting
        if self.waiting:
            _dict['waiting'] = self.waiting.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ContainerState:
        """Create an instance of V1ContainerState from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ContainerState.parse_obj(obj)

        _obj = V1ContainerState.parse_obj({
            "running": V1ContainerStateRunning.from_dict(obj.get("running")) if obj.get("running") is not None else None,
            "terminated": V1ContainerStateTerminated.from_dict(obj.get("terminated")) if obj.get("terminated") is not None else None,
            "waiting": V1ContainerStateWaiting.from_dict(obj.get("waiting")) if obj.get("waiting") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist
from .v1_local_object_reference import V1LocalObjectReference
from .v1_object_meta import V1ObjectMeta
from .v1_object_reference import V1ObjectReference

class V1ServiceAccount(BaseModel):
    """
    ServiceAccount binds together: * a name, understood by users, and perhaps by peripheral systems, for an identity * a principal that can be authenticated and authorized * a set of secrets  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    automount_service_account_token: Optional[StrictBool] = Field(None, alias="automountServiceAccountToken", description="AutomountServiceAccountToken indicates whether pods running as this service account should have an API token automatically mounted. Can be overridden at the pod level.")
    image_pull_secrets: Optional[conlist(V1LocalObjectReference)] = Field(None, alias="imagePullSecrets", description="ImagePullSecrets is a list of references to secrets in the same namespace to use for pulling any images in pods that reference this ServiceAccount. ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod, but ImagePullSecrets are only accessed by the kubelet. More info: https://kubernetes.io/docs/concepts/containers/images/#specifying-imagepullsecrets-on-a-pod")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    secrets: Optional[conlist(V1ObjectReference)] = Field(None, description="Secrets is a list of the secrets in the same namespace that pods running using this ServiceAccount are allowed to use. Pods are only limited to this list if this service account has a \"kubernetes.io/enforce-mountable-secrets\" annotation set to \"true\". This field should not be used to find auto-generated service account token secrets for use outside of pods. Instead, tokens can be requested directly using the TokenRequest API, or service account token secrets can be manually created. More info: https://kubernetes.io/docs/concepts/configuration/secret")
    __properties = ["apiVersion", "automountServiceAccountToken", "imagePullSecrets", "kind", "metadata", "secrets"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ServiceAccount:
        """Create an instance of V1ServiceAccount from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in image_pull_secrets (list)
        _items = []
        if self.image_pull_secrets:
            for _item in self.image_pull_secrets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['imagePullSecrets'] = _items
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in secrets (list)
        _items = []
        if self.secrets:
            for _item in self.secrets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['secrets'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ServiceAccount:
        """Create an instance of V1ServiceAccount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ServiceAccount.parse_obj(obj)

        _obj = V1ServiceAccount.parse_obj({
            "api_version": obj.get("apiVersion"),
            "automount_service_account_token": obj.get("automountServiceAccountToken"),
            "image_pull_secrets": [V1LocalObjectReference.from_dict(_item) for _item in obj.get("imagePullSecrets")] if obj.get("imagePullSecrets") is not None else None,
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "secrets": [V1ObjectReference.from_dict(_item) for _item in obj.get("secrets")] if obj.get("secrets") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_label_selector import V1LabelSelector

class V1alpha1ParamRef(BaseModel):
    """
    ParamRef describes how to locate the params to be used as input to expressions of rules applied by a policy binding.  # noqa: E501
    """
    name: Optional[StrictStr] = Field(None, description="`name` is the name of the resource being referenced.  `name` and `selector` are mutually exclusive properties. If one is set, the other must be unset.")
    namespace: Optional[StrictStr] = Field(None, description="namespace is the namespace of the referenced resource. Allows limiting the search for params to a specific namespace. Applies to both `name` and `selector` fields.  A per-namespace parameter may be used by specifying a namespace-scoped `paramKind` in the policy and leaving this field empty.  - If `paramKind` is cluster-scoped, this field MUST be unset. Setting this field results in a configuration error.  - If `paramKind` is namespace-scoped, the namespace of the object being evaluated for admission will be used when this field is left unset. Take care that if this is left empty the binding must not match any cluster-scoped resources, which will result in an error.")
    parameter_not_found_action: Optional[StrictStr] = Field(None, alias="parameterNotFoundAction", description="`parameterNotFoundAction` controls the behavior of the binding when the resource exists, and name or selector is valid, but there are no parameters matched by the binding. If the value is set to `Allow`, then no matched parameters will be treated as successful validation by the binding. If set to `Deny`, then no matched parameters will be subject to the `failurePolicy` of the policy.  Allowed values are `Allow` or `Deny` Default to `Deny`")
    selector: Optional[V1LabelSelector] = None
    __properties = ["name", "namespace", "parameterNotFoundAction", "selector"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha1ParamRef:
        """Create an instance of V1alpha1ParamRef from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of selector
        if self.selector:
            _dict['selector'] = self.selector.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha1ParamRef:
        """Create an instance of V1alpha1ParamRef from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha1ParamRef.parse_obj(obj)

        _obj = V1alpha1ParamRef.parse_obj({
            "name": obj.get("name"),
            "namespace": obj.get("namespace"),
            "parameter_not_found_action": obj.get("parameterNotFoundAction"),
            "selector": V1LabelSelector.from_dict(obj.get("selector")) if obj.get("selector") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_config_map_env_source import V1ConfigMapEnvSource
from .v1_secret_env_source import V1SecretEnvSource

class V1EnvFromSource(BaseModel):
    """
    EnvFromSource represents the source of a set of ConfigMaps  # noqa: E501
    """
    config_map_ref: Optional[V1ConfigMapEnvSource] = Field(None, alias="configMapRef")
    prefix: Optional[StrictStr] = Field(None, description="An optional identifier to prepend to each key in the ConfigMap. Must be a C_IDENTIFIER.")
    secret_ref: Optional[V1SecretEnvSource] = Field(None, alias="secretRef")
    __properties = ["configMapRef", "prefix", "secretRef"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1EnvFromSource:
        """Create an instance of V1EnvFromSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of config_map_ref
        if self.config_map_ref:
            _dict['configMapRef'] = self.config_map_ref.to_dict()
        # override the default output from pydantic by calling `to_dict()` of secret_ref
        if self.secret_ref:
            _dict['secretRef'] = self.secret_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1EnvFromSource:
        """Create an instance of V1EnvFromSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1EnvFromSource.parse_obj(obj)

        _obj = V1EnvFromSource.parse_obj({
            "config_map_ref": V1ConfigMapEnvSource.from_dict(obj.get("configMapRef")) if obj.get("configMapRef") is not None else None,
            "prefix": obj.get("prefix"),
            "secret_ref": V1SecretEnvSource.from_dict(obj.get("secretRef")) if obj.get("secretRef") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_attached_volume import V1AttachedVolume
from .v1_container_image import V1ContainerImage
from .v1_node_address import V1NodeAddress
from .v1_node_condition import V1NodeCondition
from .v1_node_config_status import V1NodeConfigStatus
from .v1_node_daemon_endpoints import V1NodeDaemonEndpoints
from .v1_node_system_info import V1NodeSystemInfo

class V1NodeStatus(BaseModel):
    """
    NodeStatus is information about the current status of a node.  # noqa: E501
    """
    addresses: Optional[conlist(V1NodeAddress)] = Field(None, description="List of addresses reachable to the node. Queried from cloud provider, if available. More info: https://kubernetes.io/docs/concepts/nodes/node/#addresses Note: This field is declared as mergeable, but the merge key is not sufficiently unique, which can cause data corruption when it is merged. Callers should instead use a full-replacement patch. See https://pr.k8s.io/79391 for an example. Consumers should assume that addresses can change during the lifetime of a Node. However, there are some exceptions where this may not be possible, such as Pods that inherit a Node's address in its own status or consumers of the downward API (status.hostIP).")
    allocatable: Optional[Dict[str, StrictStr]] = Field(None, description="Allocatable represents the resources of a node that are available for scheduling. Defaults to Capacity.")
    capacity: Optional[Dict[str, StrictStr]] = Field(None, description="Capacity represents the total resources of a node. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#capacity")
    conditions: Optional[conlist(V1NodeCondition)] = Field(None, description="Conditions is an array of current observed node conditions. More info: https://kubernetes.io/docs/concepts/nodes/node/#condition")
    config: Optional[V1NodeConfigStatus] = None
    daemon_endpoints: Optional[V1NodeDaemonEndpoints] = Field(None, alias="daemonEndpoints")
    images: Optional[conlist(V1ContainerImage)] = Field(None, description="List of container images on this node")
    node_info: Optional[V1NodeSystemInfo] = Field(None, alias="nodeInfo")
    phase: Optional[StrictStr] = Field(None, description="NodePhase is the recently observed lifecycle phase of the node. More info: https://kubernetes.io/docs/concepts/nodes/node/#phase The field is never populated, and now is deprecated.")
    volumes_attached: Optional[conlist(V1AttachedVolume)] = Field(None, alias="volumesAttached", description="List of volumes that are attached to the node.")
    volumes_in_use: Optional[conlist(StrictStr)] = Field(None, alias="volumesInUse", description="List of attachable volumes in use (mounted) by the node.")
    __properties = ["addresses", "allocatable", "capacity", "conditions", "config", "daemonEndpoints", "images", "nodeInfo", "phase", "volumesAttached", "volumesInUse"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1NodeStatus:
        """Create an instance of V1NodeStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in addresses (list)
        _items = []
        if self.addresses:
            for _item in self.addresses:
                if _item:
                    _items.append(_item.to_dict())
            _dict['addresses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in conditions (list)
        _items = []
        if self.conditions:
            for _item in self.conditions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['conditions'] = _items
        # override the default output from pydantic by calling `to_dict()` of config
        if self.config:
            _dict['config'] = self.config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of daemon_endpoints
        if self.daemon_endpoints:
            _dict['daemonEndpoints'] = self.daemon_endpoints.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in images (list)
        _items = []
        if self.images:
            for _item in self.images:
                if _item:
                    _items.append(_item.to_dict())
            _dict['images'] = _items
        # override the default output from pydantic by calling `to_dict()` of node_info
        if self.node_info:
            _dict['nodeInfo'] = self.node_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in volumes_attached (list)
        _items = []
        if self.volumes_attached:
            for _item in self.volumes_attached:
                if _item:
                    _items.append(_item.to_dict())
            _dict['volumesAttached'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1NodeStatus:
        """Create an instance of V1NodeStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1NodeStatus.parse_obj(obj)

        _obj = V1NodeStatus.parse_obj({
            "addresses": [V1NodeAddress.from_dict(_item) for _item in obj.get("addresses")] if obj.get("addresses") is not None else None,
            "allocatable": obj.get("allocatable"),
            "capacity": obj.get("capacity"),
            "conditions": [V1NodeCondition.from_dict(_item) for _item in obj.get("conditions")] if obj.get("conditions") is not None else None,
            "config": V1NodeConfigStatus.from_dict(obj.get("config")) if obj.get("config") is not None else None,
            "daemon_endpoints": V1NodeDaemonEndpoints.from_dict(obj.get("daemonEndpoints")) if obj.get("daemonEndpoints") is not None else None,
            "images": [V1ContainerImage.from_dict(_item) for _item in obj.get("images")] if obj.get("images") is not None else None,
            "node_info": V1NodeSystemInfo.from_dict(obj.get("nodeInfo")) if obj.get("nodeInfo") is not None else None,
            "phase": obj.get("phase"),
            "volumes_attached": [V1AttachedVolume.from_dict(_item) for _item in obj.get("volumesAttached")] if obj.get("volumesAttached") is not None else None,
            "volumes_in_use": obj.get("volumesInUse")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_csi_node_spec import V1CSINodeSpec
from .v1_object_meta import V1ObjectMeta

class V1CSINode(BaseModel):
    """
    CSINode holds information about all CSI drivers installed on a node. CSI drivers do not need to create the CSINode object directly. As long as they use the node-driver-registrar sidecar container, the kubelet will automatically populate the CSINode object for the CSI driver as part of kubelet plugin registration. CSINode has the same name as a node. If the object is missing, it means either there are no CSI Drivers available on the node, or the Kubelet version is low enough that it doesn't create this object. CSINode has an OwnerReference that points to the corresponding node object.  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    spec: V1CSINodeSpec = Field(...)
    __properties = ["apiVersion", "kind", "metadata", "spec"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1CSINode:
        """Create an instance of V1CSINode from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict['spec'] = self.spec.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1CSINode:
        """Create an instance of V1CSINode from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1CSINode.parse_obj(obj)

        _obj = V1CSINode.parse_obj({
            "api_version": obj.get("apiVersion"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "spec": V1CSINodeSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_ingress_backend import V1IngressBackend
from .v1_ingress_rule import V1IngressRule
from .v1_ingress_tls import V1IngressTLS

class V1IngressSpec(BaseModel):
    """
    IngressSpec describes the Ingress the user wishes to exist.  # noqa: E501
    """
    default_backend: Optional[V1IngressBackend] = Field(None, alias="defaultBackend")
    ingress_class_name: Optional[StrictStr] = Field(None, alias="ingressClassName", description="ingressClassName is the name of an IngressClass cluster resource. Ingress controller implementations use this field to know whether they should be serving this Ingress resource, by a transitive connection (controller -> IngressClass -> Ingress resource). Although the `kubernetes.io/ingress.class` annotation (simple constant name) was never formally defined, it was widely supported by Ingress controllers to create a direct binding between Ingress controller and Ingress resources. Newly created Ingress resources should prefer using the field. However, even though the annotation is officially deprecated, for backwards compatibility reasons, ingress controllers should still honor that annotation if present.")
    rules: Optional[conlist(V1IngressRule)] = Field(None, description="rules is a list of host rules used to configure the Ingress. If unspecified, or no rule matches, all traffic is sent to the default backend.")
    tls: Optional[conlist(V1IngressTLS)] = Field(None, description="tls represents the TLS configuration. Currently the Ingress only supports a single TLS port, 443. If multiple members of this list specify different hosts, they will be multiplexed on the same port according to the hostname specified through the SNI TLS extension, if the ingress controller fulfilling the ingress supports SNI.")
    __properties = ["defaultBackend", "ingressClassName", "rules", "tls"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1IngressSpec:
        """Create an instance of V1IngressSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of default_backend
        if self.default_backend:
            _dict['defaultBackend'] = self.default_backend.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item in self.rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['rules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tls (list)
        _items = []
        if self.tls:
            for _item in self.tls:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tls'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1IngressSpec:
        """Create an instance of V1IngressSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1IngressSpec.parse_obj(obj)

        _obj = V1IngressSpec.parse_obj({
            "default_backend": V1IngressBackend.from_dict(obj.get("defaultBackend")) if obj.get("defaultBackend") is not None else None,
            "ingress_class_name": obj.get("ingressClassName"),
            "rules": [V1IngressRule.from_dict(_item) for _item in obj.get("rules")] if obj.get("rules") is not None else None,
            "tls": [V1IngressTLS.from_dict(_item) for _item in obj.get("tls")] if obj.get("tls") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_object_meta import V1ObjectMeta
from .v1alpha1_cluster_trust_bundle_spec import V1alpha1ClusterTrustBundleSpec

class V1alpha1ClusterTrustBundle(BaseModel):
    """
    ClusterTrustBundle is a cluster-scoped container for X.509 trust anchors (root certificates).  ClusterTrustBundle objects are considered to be readable by any authenticated user in the cluster, because they can be mounted by pods using the `clusterTrustBundle` projection.  All service accounts have read access to ClusterTrustBundles by default.  Users who only have namespace-level access to a cluster can read ClusterTrustBundles by impersonating a serviceaccount that they have access to.  It can be optionally associated with a particular assigner, in which case it contains one valid set of trust anchors for that signer. Signers may have multiple associated ClusterTrustBundles; each is an independent set of trust anchors for that signer. Admission control is used to enforce that only users with permissions on the signer can create or modify the corresponding bundle.  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    spec: V1alpha1ClusterTrustBundleSpec = Field(...)
    __properties = ["apiVersion", "kind", "metadata", "spec"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha1ClusterTrustBundle:
        """Create an instance of V1alpha1ClusterTrustBundle from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict['spec'] = self.spec.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha1ClusterTrustBundle:
        """Create an instance of V1alpha1ClusterTrustBundle from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha1ClusterTrustBundle.parse_obj(obj)

        _obj = V1alpha1ClusterTrustBundle.parse_obj({
            "api_version": obj.get("apiVersion"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "spec": V1alpha1ClusterTrustBundleSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v2_hpa_scaling_rules import V2HPAScalingRules

class V2HorizontalPodAutoscalerBehavior(BaseModel):
    """
    HorizontalPodAutoscalerBehavior configures the scaling behavior of the target in both Up and Down directions (scaleUp and scaleDown fields respectively).  # noqa: E501
    """
    scale_down: Optional[V2HPAScalingRules] = Field(None, alias="scaleDown")
    scale_up: Optional[V2HPAScalingRules] = Field(None, alias="scaleUp")
    __properties = ["scaleDown", "scaleUp"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V2HorizontalPodAutoscalerBehavior:
        """Create an instance of V2HorizontalPodAutoscalerBehavior from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of scale_down
        if self.scale_down:
            _dict['scaleDown'] = self.scale_down.to_dict()
        # override the default output from pydantic by calling `to_dict()` of scale_up
        if self.scale_up:
            _dict['scaleUp'] = self.scale_up.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V2HorizontalPodAutoscalerBehavior:
        """Create an instance of V2HorizontalPodAutoscalerBehavior from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V2HorizontalPodAutoscalerBehavior.parse_obj(obj)

        _obj = V2HorizontalPodAutoscalerBehavior.parse_obj({
            "scale_down": V2HPAScalingRules.from_dict(obj.get("scaleDown")) if obj.get("scaleDown") is not None else None,
            "scale_up": V2HPAScalingRules.from_dict(obj.get("scaleUp")) if obj.get("scaleUp") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from .v2_hpa_scaling_policy import V2HPAScalingPolicy

class V2HPAScalingRules(BaseModel):
    """
    HPAScalingRules configures the scaling behavior for one direction. These Rules are applied after calculating DesiredReplicas from metrics for the HPA. They can limit the scaling velocity by specifying scaling policies. They can prevent flapping by specifying the stabilization window, so that the number of replicas is not set instantly, instead, the safest value from the stabilization window is chosen.  # noqa: E501
    """
    policies: Optional[conlist(V2HPAScalingPolicy)] = Field(None, description="policies is a list of potential scaling polices which can be used during scaling. At least one policy must be specified, otherwise the HPAScalingRules will be discarded as invalid")
    select_policy: Optional[StrictStr] = Field(None, alias="selectPolicy", description="selectPolicy is used to specify which policy should be used. If not set, the default value Max is used.")
    stabilization_window_seconds: Optional[StrictInt] = Field(None, alias="stabilizationWindowSeconds", description="stabilizationWindowSeconds is the number of seconds for which past recommendations should be considered while scaling up or scaling down. StabilizationWindowSeconds must be greater than or equal to zero and less than or equal to 3600 (one hour). If not set, use the default values: - For scale up: 0 (i.e. no stabilization is done). - For scale down: 300 (i.e. the stabilization window is 300 seconds long).")
    __properties = ["policies", "selectPolicy", "stabilizationWindowSeconds"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V2HPAScalingRules:
        """Create an instance of V2HPAScalingRules from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in policies (list)
        _items = []
        if self.policies:
            for _item in self.policies:
                if _item:
                    _items.append(_item.to_dict())
            _dict['policies'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V2HPAScalingRules:
        """Create an instance of V2HPAScalingRules from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V2HPAScalingRules.parse_obj(obj)

        _obj = V2HPAScalingRules.parse_obj({
            "policies": [V2HPAScalingPolicy.from_dict(_item) for _item in obj.get("policies")] if obj.get("policies") is not None else None,
            "select_policy": obj.get("selectPolicy"),
            "stabilization_window_seconds": obj.get("stabilizationWindowSeconds")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_label_selector import V1LabelSelector

class V1PodDisruptionBudgetSpec(BaseModel):
    """
    PodDisruptionBudgetSpec is a description of a PodDisruptionBudget.  # noqa: E501
    """
    max_unavailable: Optional[Dict[str, Any]] = Field(None, alias="maxUnavailable", description="An eviction is allowed if at most \"maxUnavailable\" pods selected by \"selector\" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with \"minAvailable\".")
    min_available: Optional[Dict[str, Any]] = Field(None, alias="minAvailable", description="An eviction is allowed if at least \"minAvailable\" pods selected by \"selector\" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying \"100%\".")
    selector: Optional[V1LabelSelector] = None
    unhealthy_pod_eviction_policy: Optional[StrictStr] = Field(None, alias="unhealthyPodEvictionPolicy", description="UnhealthyPodEvictionPolicy defines the criteria for when unhealthy pods should be considered for eviction. Current implementation considers healthy pods, as pods that have status.conditions item with type=\"Ready\",status=\"True\".  Valid policies are IfHealthyBudget and AlwaysAllow. If no policy is specified, the default behavior will be used, which corresponds to the IfHealthyBudget policy.  IfHealthyBudget policy means that running pods (status.phase=\"Running\"), but not yet healthy can be evicted only if the guarded application is not disrupted (status.currentHealthy is at least equal to status.desiredHealthy). Healthy pods will be subject to the PDB for eviction.  AlwaysAllow policy means that all running pods (status.phase=\"Running\"), but not yet healthy are considered disrupted and can be evicted regardless of whether the criteria in a PDB is met. This means perspective running pods of a disrupted application might not get a chance to become healthy. Healthy pods will be subject to the PDB for eviction.  Additional policies may be added in the future. Clients making eviction decisions should disallow eviction of unhealthy pods if they encounter an unrecognized policy in this field.  This field is beta-level. The eviction API uses this field when the feature gate PDBUnhealthyPodEvictionPolicy is enabled (enabled by default).")
    __properties = ["maxUnavailable", "minAvailable", "selector", "unhealthyPodEvictionPolicy"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodDisruptionBudgetSpec:
        """Create an instance of V1PodDisruptionBudgetSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of selector
        if self.selector:
            _dict['selector'] = self.selector.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodDisruptionBudgetSpec:
        """Create an instance of V1PodDisruptionBudgetSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodDisruptionBudgetSpec.parse_obj(obj)

        _obj = V1PodDisruptionBudgetSpec.parse_obj({
            "max_unavailable": obj.get("maxUnavailable"),
            "min_available": obj.get("minAvailable"),
            "selector": V1LabelSelector.from_dict(obj.get("selector")) if obj.get("selector") is not None else None,
            "unhealthy_pod_eviction_policy": obj.get("unhealthyPodEvictionPolicy")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class V1CertificateSigningRequestCondition(BaseModel):
    """
    CertificateSigningRequestCondition describes a condition of a CertificateSigningRequest object  # noqa: E501
    """
    last_transition_time: Optional[datetime] = Field(None, alias="lastTransitionTime", description="lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.")
    last_update_time: Optional[datetime] = Field(None, alias="lastUpdateTime", description="lastUpdateTime is the time of the last update to this condition")
    message: Optional[StrictStr] = Field(None, description="message contains a human readable message with details about the request state")
    reason: Optional[StrictStr] = Field(None, description="reason indicates a brief reason for the request state")
    status: StrictStr = Field(..., description="status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be \"False\" or \"Unknown\".")
    type: StrictStr = Field(..., description="type of the condition. Known conditions are \"Approved\", \"Denied\", and \"Failed\".  An \"Approved\" condition is added via the /approval subresource, indicating the request was approved and should be issued by the signer.  A \"Denied\" condition is added via the /approval subresource, indicating the request was denied and should not be issued by the signer.  A \"Failed\" condition is added via the /status subresource, indicating the signer failed to issue the certificate.  Approved and Denied conditions are mutually exclusive. Approved, Denied, and Failed conditions cannot be removed once added.  Only one condition of a given type is allowed.")
    __properties = ["lastTransitionTime", "lastUpdateTime", "message", "reason", "status", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1CertificateSigningRequestCondition:
        """Create an instance of V1CertificateSigningRequestCondition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1CertificateSigningRequestCondition:
        """Create an instance of V1CertificateSigningRequestCondition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1CertificateSigningRequestCondition.parse_obj(obj)

        _obj = V1CertificateSigningRequestCondition.parse_obj({
            "last_transition_time": obj.get("lastTransitionTime"),
            "last_update_time": obj.get("lastUpdateTime"),
            "message": obj.get("message"),
            "reason": obj.get("reason"),
            "status": obj.get("status"),
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, conlist
from .v1_network_policy_peer import V1NetworkPolicyPeer
from .v1_network_policy_port import V1NetworkPolicyPort

class V1NetworkPolicyIngressRule(BaseModel):
    """
    NetworkPolicyIngressRule describes a particular set of traffic that is allowed to the pods matched by a NetworkPolicySpec's podSelector. The traffic must match both ports and from.  # noqa: E501
    """
    var_from: Optional[conlist(V1NetworkPolicyPeer)] = Field(None, alias="from", description="from is a list of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the from list.")
    ports: Optional[conlist(V1NetworkPolicyPort)] = Field(None, description="ports is a list of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.")
    __properties = ["from", "ports"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1NetworkPolicyIngressRule:
        """Create an instance of V1NetworkPolicyIngressRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in var_from (list)
        _items = []
        if self.var_from:
            for _item in self.var_from:
                if _item:
                    _items.append(_item.to_dict())
            _dict['from'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in ports (list)
        _items = []
        if self.ports:
            for _item in self.ports:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ports'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1NetworkPolicyIngressRule:
        """Create an instance of V1NetworkPolicyIngressRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1NetworkPolicyIngressRule.parse_obj(obj)

        _obj = V1NetworkPolicyIngressRule.parse_obj({
            "var_from": [V1NetworkPolicyPeer.from_dict(_item) for _item in obj.get("from")] if obj.get("from") is not None else None,
            "ports": [V1NetworkPolicyPort.from_dict(_item) for _item in obj.get("ports")] if obj.get("ports") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictStr

class V1alpha1AuditAnnotation(BaseModel):
    """
    AuditAnnotation describes how to produce an audit annotation for an API request.  # noqa: E501
    """
    key: StrictStr = Field(..., description="key specifies the audit annotation key. The audit annotation keys of a ValidatingAdmissionPolicy must be unique. The key must be a qualified name ([A-Za-z0-9][-A-Za-z0-9_.]*) no more than 63 bytes in length.  The key is combined with the resource name of the ValidatingAdmissionPolicy to construct an audit annotation key: \"{ValidatingAdmissionPolicy name}/{key}\".  If an admission webhook uses the same resource name as this ValidatingAdmissionPolicy and the same audit annotation key, the annotation key will be identical. In this case, the first annotation written with the key will be included in the audit event and all subsequent annotations with the same key will be discarded.  Required.")
    value_expression: StrictStr = Field(..., alias="valueExpression", description="valueExpression represents the expression which is evaluated by CEL to produce an audit annotation value. The expression must evaluate to either a string or null value. If the expression evaluates to a string, the audit annotation is included with the string value. If the expression evaluates to null or empty string the audit annotation will be omitted. The valueExpression may be no longer than 5kb in length. If the result of the valueExpression is more than 10kb in length, it will be truncated to 10kb.  If multiple ValidatingAdmissionPolicyBinding resources match an API request, then the valueExpression will be evaluated for each binding. All unique values produced by the valueExpressions will be joined together in a comma-separated list.  Required.")
    __properties = ["key", "valueExpression"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha1AuditAnnotation:
        """Create an instance of V1alpha1AuditAnnotation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha1AuditAnnotation:
        """Create an instance of V1alpha1AuditAnnotation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha1AuditAnnotation.parse_obj(obj)

        _obj = V1alpha1AuditAnnotation.parse_obj({
            "key": obj.get("key"),
            "value_expression": obj.get("valueExpression")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, conlist
from .v1_volume_projection import V1VolumeProjection

class V1ProjectedVolumeSource(BaseModel):
    """
    Represents a projected volume source  # noqa: E501
    """
    default_mode: Optional[StrictInt] = Field(None, alias="defaultMode", description="defaultMode are the mode bits used to set permissions on created files by default. Must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.")
    sources: Optional[conlist(V1VolumeProjection)] = Field(None, description="sources is the list of volume projections")
    __properties = ["defaultMode", "sources"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ProjectedVolumeSource:
        """Create an instance of V1ProjectedVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item in self.sources:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sources'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ProjectedVolumeSource:
        """Create an instance of V1ProjectedVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ProjectedVolumeSource.parse_obj(obj)

        _obj = V1ProjectedVolumeSource.parse_obj({
            "default_mode": obj.get("defaultMode"),
            "sources": [V1VolumeProjection.from_dict(_item) for _item in obj.get("sources")] if obj.get("sources") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_ingress_backend import V1IngressBackend

class V1HTTPIngressPath(BaseModel):
    """
    HTTPIngressPath associates a path with a backend. Incoming urls matching the path are forwarded to the backend.  # noqa: E501
    """
    backend: V1IngressBackend = Field(...)
    path: Optional[StrictStr] = Field(None, description="path is matched against the path of an incoming request. Currently it can contain characters disallowed from the conventional \"path\" part of a URL as defined by RFC 3986. Paths must begin with a '/' and must be present when using PathType with value \"Exact\" or \"Prefix\".")
    path_type: StrictStr = Field(..., alias="pathType", description="pathType determines the interpretation of the path matching. PathType can be one of the following values: * Exact: Matches the URL path exactly. * Prefix: Matches based on a URL path prefix split by '/'. Matching is   done on a path element by element basis. A path element refers is the   list of labels in the path split by the '/' separator. A request is a   match for path p if every p is an element-wise prefix of p of the   request path. Note that if the last element of the path is a substring   of the last element in request path, it is not a match (e.g. /foo/bar   matches /foo/bar/baz, but does not match /foo/barbaz). * ImplementationSpecific: Interpretation of the Path matching is up to   the IngressClass. Implementations can treat this as a separate PathType   or treat it identically to Prefix or Exact path types. Implementations are required to support all path types.")
    __properties = ["backend", "path", "pathType"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1HTTPIngressPath:
        """Create an instance of V1HTTPIngressPath from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of backend
        if self.backend:
            _dict['backend'] = self.backend.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1HTTPIngressPath:
        """Create an instance of V1HTTPIngressPath from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1HTTPIngressPath.parse_obj(obj)

        _obj = V1HTTPIngressPath.parse_obj({
            "backend": V1IngressBackend.from_dict(obj.get("backend")) if obj.get("backend") is not None else None,
            "path": obj.get("path"),
            "path_type": obj.get("pathType")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, conlist
from .v2_cross_version_object_reference import V2CrossVersionObjectReference
from .v2_horizontal_pod_autoscaler_behavior import V2HorizontalPodAutoscalerBehavior
from .v2_metric_spec import V2MetricSpec

class V2HorizontalPodAutoscalerSpec(BaseModel):
    """
    HorizontalPodAutoscalerSpec describes the desired functionality of the HorizontalPodAutoscaler.  # noqa: E501
    """
    behavior: Optional[V2HorizontalPodAutoscalerBehavior] = None
    max_replicas: StrictInt = Field(..., alias="maxReplicas", description="maxReplicas is the upper limit for the number of replicas to which the autoscaler can scale up. It cannot be less that minReplicas.")
    metrics: Optional[conlist(V2MetricSpec)] = Field(None, description="metrics contains the specifications for which to use to calculate the desired replica count (the maximum replica count across all metrics will be used).  The desired replica count is calculated multiplying the ratio between the target value and the current value by the current number of pods.  Ergo, metrics used must decrease as the pod count is increased, and vice-versa.  See the individual metric source types for more information about how each type of metric must respond. If not set, the default metric will be set to 80% average CPU utilization.")
    min_replicas: Optional[StrictInt] = Field(None, alias="minReplicas", description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.")
    scale_target_ref: V2CrossVersionObjectReference = Field(..., alias="scaleTargetRef")
    __properties = ["behavior", "maxReplicas", "metrics", "minReplicas", "scaleTargetRef"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V2HorizontalPodAutoscalerSpec:
        """Create an instance of V2HorizontalPodAutoscalerSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of behavior
        if self.behavior:
            _dict['behavior'] = self.behavior.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in metrics (list)
        _items = []
        if self.metrics:
            for _item in self.metrics:
                if _item:
                    _items.append(_item.to_dict())
            _dict['metrics'] = _items
        # override the default output from pydantic by calling `to_dict()` of scale_target_ref
        if self.scale_target_ref:
            _dict['scaleTargetRef'] = self.scale_target_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V2HorizontalPodAutoscalerSpec:
        """Create an instance of V2HorizontalPodAutoscalerSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V2HorizontalPodAutoscalerSpec.parse_obj(obj)

        _obj = V2HorizontalPodAutoscalerSpec.parse_obj({
            "behavior": V2HorizontalPodAutoscalerBehavior.from_dict(obj.get("behavior")) if obj.get("behavior") is not None else None,
            "max_replicas": obj.get("maxReplicas"),
            "metrics": [V2MetricSpec.from_dict(_item) for _item in obj.get("metrics")] if obj.get("metrics") is not None else None,
            "min_replicas": obj.get("minReplicas"),
            "scale_target_ref": V2CrossVersionObjectReference.from_dict(obj.get("scaleTargetRef")) if obj.get("scaleTargetRef") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist
from .v1_non_resource_rule import V1NonResourceRule
from .v1_resource_rule import V1ResourceRule

class V1SubjectRulesReviewStatus(BaseModel):
    """
    SubjectRulesReviewStatus contains the result of a rules check. This check can be incomplete depending on the set of authorizers the server is configured with and any errors experienced during evaluation. Because authorization rules are additive, if a rule appears in a list it's safe to assume the subject has that permission, even if that list is incomplete.  # noqa: E501
    """
    evaluation_error: Optional[StrictStr] = Field(None, alias="evaluationError", description="EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.")
    incomplete: StrictBool = Field(..., description="Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.")
    non_resource_rules: conlist(V1NonResourceRule) = Field(..., alias="nonResourceRules", description="NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.")
    resource_rules: conlist(V1ResourceRule) = Field(..., alias="resourceRules", description="ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.")
    __properties = ["evaluationError", "incomplete", "nonResourceRules", "resourceRules"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1SubjectRulesReviewStatus:
        """Create an instance of V1SubjectRulesReviewStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in non_resource_rules (list)
        _items = []
        if self.non_resource_rules:
            for _item in self.non_resource_rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['nonResourceRules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in resource_rules (list)
        _items = []
        if self.resource_rules:
            for _item in self.resource_rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['resourceRules'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1SubjectRulesReviewStatus:
        """Create an instance of V1SubjectRulesReviewStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1SubjectRulesReviewStatus.parse_obj(obj)

        _obj = V1SubjectRulesReviewStatus.parse_obj({
            "evaluation_error": obj.get("evaluationError"),
            "incomplete": obj.get("incomplete"),
            "non_resource_rules": [V1NonResourceRule.from_dict(_item) for _item in obj.get("nonResourceRules")] if obj.get("nonResourceRules") is not None else None,
            "resource_rules": [V1ResourceRule.from_dict(_item) for _item in obj.get("resourceRules")] if obj.get("resourceRules") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr

class V1AzureFileVolumeSource(BaseModel):
    """
    AzureFile represents an Azure File Service mount on the host and bind mount to the pod.  # noqa: E501
    """
    read_only: Optional[StrictBool] = Field(None, alias="readOnly", description="readOnly defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.")
    secret_name: StrictStr = Field(..., alias="secretName", description="secretName is the  name of secret that contains Azure Storage Account Name and Key")
    share_name: StrictStr = Field(..., alias="shareName", description="shareName is the azure share Name")
    __properties = ["readOnly", "secretName", "shareName"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1AzureFileVolumeSource:
        """Create an instance of V1AzureFileVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1AzureFileVolumeSource:
        """Create an instance of V1AzureFileVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1AzureFileVolumeSource.parse_obj(obj)

        _obj = V1AzureFileVolumeSource.parse_obj({
            "read_only": obj.get("readOnly"),
            "secret_name": obj.get("secretName"),
            "share_name": obj.get("shareName")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from .v1_local_object_reference import V1LocalObjectReference

class V1ISCSIVolumeSource(BaseModel):
    """
    Represents an ISCSI disk. ISCSI volumes can only be mounted as read/write once. ISCSI volumes support ownership management and SELinux relabeling.  # noqa: E501
    """
    chap_auth_discovery: Optional[StrictBool] = Field(None, alias="chapAuthDiscovery", description="chapAuthDiscovery defines whether support iSCSI Discovery CHAP authentication")
    chap_auth_session: Optional[StrictBool] = Field(None, alias="chapAuthSession", description="chapAuthSession defines whether support iSCSI Session CHAP authentication")
    fs_type: Optional[StrictStr] = Field(None, alias="fsType", description="fsType is the filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#iscsi")
    initiator_name: Optional[StrictStr] = Field(None, alias="initiatorName", description="initiatorName is the custom iSCSI Initiator Name. If initiatorName is specified with iscsiInterface simultaneously, new iSCSI interface <target portal>:<volume name> will be created for the connection.")
    iqn: StrictStr = Field(..., description="iqn is the target iSCSI Qualified Name.")
    iscsi_interface: Optional[StrictStr] = Field(None, alias="iscsiInterface", description="iscsiInterface is the interface Name that uses an iSCSI transport. Defaults to 'default' (tcp).")
    lun: StrictInt = Field(..., description="lun represents iSCSI Target Lun number.")
    portals: Optional[conlist(StrictStr)] = Field(None, description="portals is the iSCSI Target Portal List. The portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).")
    read_only: Optional[StrictBool] = Field(None, alias="readOnly", description="readOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false.")
    secret_ref: Optional[V1LocalObjectReference] = Field(None, alias="secretRef")
    target_portal: StrictStr = Field(..., alias="targetPortal", description="targetPortal is iSCSI Target Portal. The Portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).")
    __properties = ["chapAuthDiscovery", "chapAuthSession", "fsType", "initiatorName", "iqn", "iscsiInterface", "lun", "portals", "readOnly", "secretRef", "targetPortal"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ISCSIVolumeSource:
        """Create an instance of V1ISCSIVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of secret_ref
        if self.secret_ref:
            _dict['secretRef'] = self.secret_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ISCSIVolumeSource:
        """Create an instance of V1ISCSIVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ISCSIVolumeSource.parse_obj(obj)

        _obj = V1ISCSIVolumeSource.parse_obj({
            "chap_auth_discovery": obj.get("chapAuthDiscovery"),
            "chap_auth_session": obj.get("chapAuthSession"),
            "fs_type": obj.get("fsType"),
            "initiator_name": obj.get("initiatorName"),
            "iqn": obj.get("iqn"),
            "iscsi_interface": obj.get("iscsiInterface"),
            "lun": obj.get("lun"),
            "portals": obj.get("portals"),
            "read_only": obj.get("readOnly"),
            "secret_ref": V1LocalObjectReference.from_dict(obj.get("secretRef")) if obj.get("secretRef") is not None else None,
            "target_portal": obj.get("targetPortal")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist
from .v1_external_documentation import V1ExternalDocumentation
from .v1_validation_rule import V1ValidationRule

class V1JSONSchemaProps(BaseModel):
    """
    JSONSchemaProps is a JSON-Schema following Specification Draft 4 (http://json-schema.org/).  # noqa: E501
    """
    ref: Optional[StrictStr] = Field(None, alias="$ref")
    var_schema: Optional[StrictStr] = Field(None, alias="$schema")
    additional_items: Optional[Dict[str, Any]] = Field(None, alias="additionalItems", description="JSONSchemaPropsOrBool represents JSONSchemaProps or a boolean value. Defaults to true for the boolean property.")
    additional_properties: Optional[Dict[str, Any]] = Field(None, alias="additionalProperties", description="JSONSchemaPropsOrBool represents JSONSchemaProps or a boolean value. Defaults to true for the boolean property.")
    all_of: Optional[conlist(V1JSONSchemaProps)] = Field(None, alias="allOf")
    any_of: Optional[conlist(V1JSONSchemaProps)] = Field(None, alias="anyOf")
    default: Optional[Dict[str, Any]] = Field(None, description="default is a default value for undefined object fields. Defaulting is a beta feature under the CustomResourceDefaulting feature gate. Defaulting requires spec.preserveUnknownFields to be false.")
    definitions: Optional[Dict[str, V1JSONSchemaProps]] = None
    dependencies: Optional[Dict[str, Dict[str, Any]]] = None
    description: Optional[StrictStr] = None
    enum: Optional[conlist(Dict[str, Any])] = None
    example: Optional[Dict[str, Any]] = Field(None, description="JSON represents any valid JSON value. These types are supported: bool, int64, float64, string, []interface{}, map[string]interface{} and nil.")
    exclusive_maximum: Optional[StrictBool] = Field(None, alias="exclusiveMaximum")
    exclusive_minimum: Optional[StrictBool] = Field(None, alias="exclusiveMinimum")
    external_docs: Optional[V1ExternalDocumentation] = Field(None, alias="externalDocs")
    format: Optional[StrictStr] = Field(None, description="format is an OpenAPI v3 format string. Unknown formats are ignored. The following formats are validated:  - bsonobjectid: a bson object ID, i.e. a 24 characters hex string - uri: an URI as parsed by Golang net/url.ParseRequestURI - email: an email address as parsed by Golang net/mail.ParseAddress - hostname: a valid representation for an Internet host name, as defined by RFC 1034, section 3.1 [RFC1034]. - ipv4: an IPv4 IP as parsed by Golang net.ParseIP - ipv6: an IPv6 IP as parsed by Golang net.ParseIP - cidr: a CIDR as parsed by Golang net.ParseCIDR - mac: a MAC address as parsed by Golang net.ParseMAC - uuid: an UUID that allows uppercase defined by the regex (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{12}$ - uuid3: an UUID3 that allows uppercase defined by the regex (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?3[0-9a-f]{3}-?[0-9a-f]{4}-?[0-9a-f]{12}$ - uuid4: an UUID4 that allows uppercase defined by the regex (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?4[0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}$ - uuid5: an UUID5 that allows uppercase defined by the regex (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?5[0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}$ - isbn: an ISBN10 or ISBN13 number string like \"0321751043\" or \"978-0321751041\" - isbn10: an ISBN10 number string like \"0321751043\" - isbn13: an ISBN13 number string like \"978-0321751041\" - creditcard: a credit card number defined by the regex ^(?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|6(?:011|5[0-9][0-9])[0-9]{12}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|(?:2131|1800|35\\d{3})\\d{11})$ with any non digit characters mixed in - ssn: a U.S. social security number following the regex ^\\d{3}[- ]?\\d{2}[- ]?\\d{4}$ - hexcolor: an hexadecimal color code like \"#FFFFFF: following the regex ^#?([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$ - rgbcolor: an RGB color code like rgb like \"rgb(255,255,2559\" - byte: base64 encoded binary data - password: any kind of string - date: a date string like \"2006-01-02\" as defined by full-date in RFC3339 - duration: a duration string like \"22 ns\" as parsed by Golang time.ParseDuration or compatible with Scala duration format - datetime: a date time string like \"2014-12-15T19:30:20.000Z\" as defined by date-time in RFC3339.")
    id: Optional[StrictStr] = None
    items: Optional[Dict[str, Any]] = Field(None, description="JSONSchemaPropsOrArray represents a value that can either be a JSONSchemaProps or an array of JSONSchemaProps. Mainly here for serialization purposes.")
    max_items: Optional[StrictInt] = Field(None, alias="maxItems")
    max_length: Optional[StrictInt] = Field(None, alias="maxLength")
    max_properties: Optional[StrictInt] = Field(None, alias="maxProperties")
    maximum: Optional[Union[StrictFloat, StrictInt]] = None
    min_items: Optional[StrictInt] = Field(None, alias="minItems")
    min_length: Optional[StrictInt] = Field(None, alias="minLength")
    min_properties: Optional[StrictInt] = Field(None, alias="minProperties")
    minimum: Optional[Union[StrictFloat, StrictInt]] = None
    multiple_of: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="multipleOf")
    var_not: Optional[V1JSONSchemaProps] = Field(None, alias="not")
    nullable: Optional[StrictBool] = None
    one_of: Optional[conlist(V1JSONSchemaProps)] = Field(None, alias="oneOf")
    pattern: Optional[StrictStr] = None
    pattern_properties: Optional[Dict[str, V1JSONSchemaProps]] = Field(None, alias="patternProperties")
    properties: Optional[Dict[str, V1JSONSchemaProps]] = None
    required: Optional[conlist(StrictStr)] = None
    title: Optional[StrictStr] = None
    type: Optional[StrictStr] = None
    unique_items: Optional[StrictBool] = Field(None, alias="uniqueItems")
    x_kubernetes_embedded_resource: Optional[StrictBool] = Field(None, alias="x-kubernetes-embedded-resource", description="x-kubernetes-embedded-resource defines that the value is an embedded Kubernetes runtime.Object, with TypeMeta and ObjectMeta. The type must be object. It is allowed to further restrict the embedded object. kind, apiVersion and metadata are validated automatically. x-kubernetes-preserve-unknown-fields is allowed to be true, but does not have to be if the object is fully specified (up to kind, apiVersion, metadata).")
    x_kubernetes_int_or_string: Optional[StrictBool] = Field(None, alias="x-kubernetes-int-or-string", description="x-kubernetes-int-or-string specifies that this value is either an integer or a string. If this is true, an empty type is allowed and type as child of anyOf is permitted if following one of the following patterns:  1) anyOf:    - type: integer    - type: string 2) allOf:    - anyOf:      - type: integer      - type: string    - ... zero or more")
    x_kubernetes_list_map_keys: Optional[conlist(StrictStr)] = Field(None, alias="x-kubernetes-list-map-keys", description="x-kubernetes-list-map-keys annotates an array with the x-kubernetes-list-type `map` by specifying the keys used as the index of the map.  This tag MUST only be used on lists that have the \"x-kubernetes-list-type\" extension set to \"map\". Also, the values specified for this attribute must be a scalar typed field of the child structure (no nesting is supported).  The properties specified must either be required or have a default value, to ensure those properties are present for all list items.")
    x_kubernetes_list_type: Optional[StrictStr] = Field(None, alias="x-kubernetes-list-type", description="x-kubernetes-list-type annotates an array to further describe its topology. This extension must only be used on lists and may have 3 possible values:  1) `atomic`: the list is treated as a single entity, like a scalar.      Atomic lists will be entirely replaced when updated. This extension      may be used on any type of list (struct, scalar, ...). 2) `set`:      Sets are lists that must not have multiple items with the same value. Each      value must be a scalar, an object with x-kubernetes-map-type `atomic` or an      array with x-kubernetes-list-type `atomic`. 3) `map`:      These lists are like maps in that their elements have a non-index key      used to identify them. Order is preserved upon merge. The map tag      must only be used on a list with elements of type object. Defaults to atomic for arrays.")
    x_kubernetes_map_type: Optional[StrictStr] = Field(None, alias="x-kubernetes-map-type", description="x-kubernetes-map-type annotates an object to further describe its topology. This extension must only be used when type is object and may have 2 possible values:  1) `granular`:      These maps are actual maps (key-value pairs) and each fields are independent      from each other (they can each be manipulated by separate actors). This is      the default behaviour for all maps. 2) `atomic`: the list is treated as a single entity, like a scalar.      Atomic maps will be entirely replaced when updated.")
    x_kubernetes_preserve_unknown_fields: Optional[StrictBool] = Field(None, alias="x-kubernetes-preserve-unknown-fields", description="x-kubernetes-preserve-unknown-fields stops the API server decoding step from pruning fields which are not specified in the validation schema. This affects fields recursively, but switches back to normal pruning behaviour if nested properties or additionalProperties are specified in the schema. This can either be true or undefined. False is forbidden.")
    x_kubernetes_validations: Optional[conlist(V1ValidationRule)] = Field(None, alias="x-kubernetes-validations", description="x-kubernetes-validations describes a list of validation rules written in the CEL expression language. This field is an alpha-level. Using this field requires the feature gate `CustomResourceValidationExpressions` to be enabled.")
    __properties = ["$ref", "$schema", "additionalItems", "additionalProperties", "allOf", "anyOf", "default", "definitions", "dependencies", "description", "enum", "example", "exclusiveMaximum", "exclusiveMinimum", "externalDocs", "format", "id", "items", "maxItems", "maxLength", "maxProperties", "maximum", "minItems", "minLength", "minProperties", "minimum", "multipleOf", "not", "nullable", "oneOf", "pattern", "patternProperties", "properties", "required", "title", "type", "uniqueItems", "x-kubernetes-embedded-resource", "x-kubernetes-int-or-string", "x-kubernetes-list-map-keys", "x-kubernetes-list-type", "x-kubernetes-map-type", "x-kubernetes-preserve-unknown-fields", "x-kubernetes-validations"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1JSONSchemaProps:
        """Create an instance of V1JSONSchemaProps from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in all_of (list)
        _items = []
        if self.all_of:
            for _item in self.all_of:
                if _item:
                    _items.append(_item.to_dict())
            _dict['allOf'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in any_of (list)
        _items = []
        if self.any_of:
            for _item in self.any_of:
                if _item:
                    _items.append(_item.to_dict())
            _dict['anyOf'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in definitions (dict)
        _field_dict = {}
        if self.definitions:
            for _key in self.definitions:
                if self.definitions[_key]:
                    _field_dict[_key] = self.definitions[_key].to_dict()
            _dict['definitions'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of external_docs
        if self.external_docs:
            _dict['externalDocs'] = self.external_docs.to_dict()
        # override the default output from pydantic by calling `to_dict()` of var_not
        if self.var_not:
            _dict['not'] = self.var_not.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in one_of (list)
        _items = []
        if self.one_of:
            for _item in self.one_of:
                if _item:
                    _items.append(_item.to_dict())
            _dict['oneOf'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in pattern_properties (dict)
        _field_dict = {}
        if self.pattern_properties:
            for _key in self.pattern_properties:
                if self.pattern_properties[_key]:
                    _field_dict[_key] = self.pattern_properties[_key].to_dict()
            _dict['patternProperties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in x_kubernetes_validations (list)
        _items = []
        if self.x_kubernetes_validations:
            for _item in self.x_kubernetes_validations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['x-kubernetes-validations'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1JSONSchemaProps:
        """Create an instance of V1JSONSchemaProps from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1JSONSchemaProps.parse_obj(obj)

        _obj = V1JSONSchemaProps.parse_obj({
            "ref": obj.get("$ref"),
            "var_schema": obj.get("$schema"),
            "additional_items": obj.get("additionalItems"),
            "additional_properties": obj.get("additionalProperties"),
            "all_of": [V1JSONSchemaProps.from_dict(_item) for _item in obj.get("allOf")] if obj.get("allOf") is not None else None,
            "any_of": [V1JSONSchemaProps.from_dict(_item) for _item in obj.get("anyOf")] if obj.get("anyOf") is not None else None,
            "default": obj.get("default"),
            "definitions": dict(
                (_k, V1JSONSchemaProps.from_dict(_v))
                for _k, _v in obj.get("definitions").items()
            )
            if obj.get("definitions") is not None
            else None,
            "dependencies": obj.get("dependencies"),
            "description": obj.get("description"),
            "enum": obj.get("enum"),
            "example": obj.get("example"),
            "exclusive_maximum": obj.get("exclusiveMaximum"),
            "exclusive_minimum": obj.get("exclusiveMinimum"),
            "external_docs": V1ExternalDocumentation.from_dict(obj.get("externalDocs")) if obj.get("externalDocs") is not None else None,
            "format": obj.get("format"),
            "id": obj.get("id"),
            "items": obj.get("items"),
            "max_items": obj.get("maxItems"),
            "max_length": obj.get("maxLength"),
            "max_properties": obj.get("maxProperties"),
            "maximum": obj.get("maximum"),
            "min_items": obj.get("minItems"),
            "min_length": obj.get("minLength"),
            "min_properties": obj.get("minProperties"),
            "minimum": obj.get("minimum"),
            "multiple_of": obj.get("multipleOf"),
            "var_not": V1JSONSchemaProps.from_dict(obj.get("not")) if obj.get("not") is not None else None,
            "nullable": obj.get("nullable"),
            "one_of": [V1JSONSchemaProps.from_dict(_item) for _item in obj.get("oneOf")] if obj.get("oneOf") is not None else None,
            "pattern": obj.get("pattern"),
            "pattern_properties": dict(
                (_k, V1JSONSchemaProps.from_dict(_v))
                for _k, _v in obj.get("patternProperties").items()
            )
            if obj.get("patternProperties") is not None
            else None,
            "properties": dict(
                (_k, V1JSONSchemaProps.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "required": obj.get("required"),
            "title": obj.get("title"),
            "type": obj.get("type"),
            "unique_items": obj.get("uniqueItems"),
            "x_kubernetes_embedded_resource": obj.get("x-kubernetes-embedded-resource"),
            "x_kubernetes_int_or_string": obj.get("x-kubernetes-int-or-string"),
            "x_kubernetes_list_map_keys": obj.get("x-kubernetes-list-map-keys"),
            "x_kubernetes_list_type": obj.get("x-kubernetes-list-type"),
            "x_kubernetes_map_type": obj.get("x-kubernetes-map-type"),
            "x_kubernetes_preserve_unknown_fields": obj.get("x-kubernetes-preserve-unknown-fields"),
            "x_kubernetes_validations": [V1ValidationRule.from_dict(_item) for _item in obj.get("x-kubernetes-validations")] if obj.get("x-kubernetes-validations") is not None else None
        })
        return _obj

V1JSONSchemaProps.update_forward_refs()




# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictInt

class V1HorizontalPodAutoscalerStatus(BaseModel):
    """
    current status of a horizontal pod autoscaler  # noqa: E501
    """
    current_cpu_utilization_percentage: Optional[StrictInt] = Field(None, alias="currentCPUUtilizationPercentage", description="currentCPUUtilizationPercentage is the current average CPU utilization over all pods, represented as a percentage of requested CPU, e.g. 70 means that an average pod is using now 70% of its requested CPU.")
    current_replicas: StrictInt = Field(..., alias="currentReplicas", description="currentReplicas is the current number of replicas of pods managed by this autoscaler.")
    desired_replicas: StrictInt = Field(..., alias="desiredReplicas", description="desiredReplicas is the  desired number of replicas of pods managed by this autoscaler.")
    last_scale_time: Optional[datetime] = Field(None, alias="lastScaleTime", description="lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number of pods; used by the autoscaler to control how often the number of pods is changed.")
    observed_generation: Optional[StrictInt] = Field(None, alias="observedGeneration", description="observedGeneration is the most recent generation observed by this autoscaler.")
    __properties = ["currentCPUUtilizationPercentage", "currentReplicas", "desiredReplicas", "lastScaleTime", "observedGeneration"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1HorizontalPodAutoscalerStatus:
        """Create an instance of V1HorizontalPodAutoscalerStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1HorizontalPodAutoscalerStatus:
        """Create an instance of V1HorizontalPodAutoscalerStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1HorizontalPodAutoscalerStatus.parse_obj(obj)

        _obj = V1HorizontalPodAutoscalerStatus.parse_obj({
            "current_cpu_utilization_percentage": obj.get("currentCPUUtilizationPercentage"),
            "current_replicas": obj.get("currentReplicas"),
            "desired_replicas": obj.get("desiredReplicas"),
            "last_scale_time": obj.get("lastScaleTime"),
            "observed_generation": obj.get("observedGeneration")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from .admissionregistration_v1_webhook_client_config import AdmissionregistrationV1WebhookClientConfig
from .v1_label_selector import V1LabelSelector
from .v1_match_condition import V1MatchCondition
from .v1_rule_with_operations import V1RuleWithOperations

class V1ValidatingWebhook(BaseModel):
    """
    ValidatingWebhook describes an admission webhook and the resources and operations it applies to.  # noqa: E501
    """
    admission_review_versions: conlist(StrictStr) = Field(..., alias="admissionReviewVersions", description="AdmissionReviewVersions is an ordered list of preferred `AdmissionReview` versions the Webhook expects. API server will try to use first version in the list which it supports. If none of the versions specified in this list supported by API server, validation will fail for this object. If a persisted webhook configuration specifies allowed versions and does not include any versions known to the API Server, calls to the webhook will fail and be subject to the failure policy.")
    client_config: AdmissionregistrationV1WebhookClientConfig = Field(..., alias="clientConfig")
    failure_policy: Optional[StrictStr] = Field(None, alias="failurePolicy", description="FailurePolicy defines how unrecognized errors from the admission endpoint are handled - allowed values are Ignore or Fail. Defaults to Fail.")
    match_conditions: Optional[conlist(V1MatchCondition)] = Field(None, alias="matchConditions", description="MatchConditions is a list of conditions that must be met for a request to be sent to this webhook. Match conditions filter requests that have already been matched by the rules, namespaceSelector, and objectSelector. An empty list of matchConditions matches all requests. There are a maximum of 64 match conditions allowed.  The exact matching logic is (in order):   1. If ANY matchCondition evaluates to FALSE, the webhook is skipped.   2. If ALL matchConditions evaluate to TRUE, the webhook is called.   3. If any matchCondition evaluates to an error (but none are FALSE):      - If failurePolicy=Fail, reject the request      - If failurePolicy=Ignore, the error is ignored and the webhook is skipped  This is a beta feature and managed by the AdmissionWebhookMatchConditions feature gate.")
    match_policy: Optional[StrictStr] = Field(None, alias="matchPolicy", description="matchPolicy defines how the \"rules\" list is used to match incoming requests. Allowed values are \"Exact\" or \"Equivalent\".  - Exact: match a request only if it exactly matches a specified rule. For example, if deployments can be modified via apps/v1, apps/v1beta1, and extensions/v1beta1, but \"rules\" only included `apiGroups:[\"apps\"], apiVersions:[\"v1\"], resources: [\"deployments\"]`, a request to apps/v1beta1 or extensions/v1beta1 would not be sent to the webhook.  - Equivalent: match a request if modifies a resource listed in rules, even via another API group or version. For example, if deployments can be modified via apps/v1, apps/v1beta1, and extensions/v1beta1, and \"rules\" only included `apiGroups:[\"apps\"], apiVersions:[\"v1\"], resources: [\"deployments\"]`, a request to apps/v1beta1 or extensions/v1beta1 would be converted to apps/v1 and sent to the webhook.  Defaults to \"Equivalent\"")
    name: StrictStr = Field(..., description="The name of the admission webhook. Name should be fully qualified, e.g., imagepolicy.kubernetes.io, where \"imagepolicy\" is the name of the webhook, and kubernetes.io is the name of the organization. Required.")
    namespace_selector: Optional[V1LabelSelector] = Field(None, alias="namespaceSelector")
    object_selector: Optional[V1LabelSelector] = Field(None, alias="objectSelector")
    rules: Optional[conlist(V1RuleWithOperations)] = Field(None, description="Rules describes what operations on what resources/subresources the webhook cares about. The webhook cares about an operation if it matches _any_ Rule. However, in order to prevent ValidatingAdmissionWebhooks and MutatingAdmissionWebhooks from putting the cluster in a state which cannot be recovered from without completely disabling the plugin, ValidatingAdmissionWebhooks and MutatingAdmissionWebhooks are never called on admission requests for ValidatingWebhookConfiguration and MutatingWebhookConfiguration objects.")
    side_effects: StrictStr = Field(..., alias="sideEffects", description="SideEffects states whether this webhook has side effects. Acceptable values are: None, NoneOnDryRun (webhooks created via v1beta1 may also specify Some or Unknown). Webhooks with side effects MUST implement a reconciliation system, since a request may be rejected by a future step in the admission chain and the side effects therefore need to be undone. Requests with the dryRun attribute will be auto-rejected if they match a webhook with sideEffects == Unknown or Some.")
    timeout_seconds: Optional[StrictInt] = Field(None, alias="timeoutSeconds", description="TimeoutSeconds specifies the timeout for this webhook. After the timeout passes, the webhook call will be ignored or the API call will fail based on the failure policy. The timeout value must be between 1 and 30 seconds. Default to 10 seconds.")
    __properties = ["admissionReviewVersions", "clientConfig", "failurePolicy", "matchConditions", "matchPolicy", "name", "namespaceSelector", "objectSelector", "rules", "sideEffects", "timeoutSeconds"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ValidatingWebhook:
        """Create an instance of V1ValidatingWebhook from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of client_config
        if self.client_config:
            _dict['clientConfig'] = self.client_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in match_conditions (list)
        _items = []
        if self.match_conditions:
            for _item in self.match_conditions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['matchConditions'] = _items
        # override the default output from pydantic by calling `to_dict()` of namespace_selector
        if self.namespace_selector:
            _dict['namespaceSelector'] = self.namespace_selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of object_selector
        if self.object_selector:
            _dict['objectSelector'] = self.object_selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item in self.rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['rules'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ValidatingWebhook:
        """Create an instance of V1ValidatingWebhook from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ValidatingWebhook.parse_obj(obj)

        _obj = V1ValidatingWebhook.parse_obj({
            "admission_review_versions": obj.get("admissionReviewVersions"),
            "client_config": AdmissionregistrationV1WebhookClientConfig.from_dict(obj.get("clientConfig")) if obj.get("clientConfig") is not None else None,
            "failure_policy": obj.get("failurePolicy"),
            "match_conditions": [V1MatchCondition.from_dict(_item) for _item in obj.get("matchConditions")] if obj.get("matchConditions") is not None else None,
            "match_policy": obj.get("matchPolicy"),
            "name": obj.get("name"),
            "namespace_selector": V1LabelSelector.from_dict(obj.get("namespaceSelector")) if obj.get("namespaceSelector") is not None else None,
            "object_selector": V1LabelSelector.from_dict(obj.get("objectSelector")) if obj.get("objectSelector") is not None else None,
            "rules": [V1RuleWithOperations.from_dict(_item) for _item in obj.get("rules")] if obj.get("rules") is not None else None,
            "side_effects": obj.get("sideEffects"),
            "timeout_seconds": obj.get("timeoutSeconds")
        })
        return _obj


